namespace tadah_fussy
{
  namespace statistics
  {
    namespace regression
    {
      /*
        NIPALS コンストラクタ : 独立変数 X と従属変数 y を指定して構築
      */
      template< class T, class U >
      NIPALS::NIPALS( const Matrix< T >& x, const Matrix< U >& y )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::regression::NIPALS constructor";

        if ( x.rows() != y.rows() )
          throw exception::NotEqualLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : data x and y must be same row size.", x.rows(), y.rows() );
        if ( x.rows() == 0 )
          throw exception::NotPositiveLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : row size of x and y must be more than zero.", x.rows() );
        if ( x.cols() == 0 )
          throw exception::NotPositiveLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : column size of x must be more than zero.", x.cols() );
        if ( y.cols() == 0 )
          throw exception::NotPositiveLength< typename Matrix< U >::size_type >
            ( FUNC_NAME + " : column size of y must be more than zero.", y.cols() );

        x_.assign( x.begin(), x.end(), x.rows(), true );
        y_.assign( y.begin(), y.end(), y.rows(), true );

        for ( Matrix< stat_type >::size_type c = 0 ; c < x_.cols() ; ++c ) {
          Matrix< stat_type >::iterator col = x_.column( c );
          Normalize( col, col.end() );
        }
        for ( Matrix< stat_type >::size_type c = 0 ; c < y_.cols() ; ++c ) {
          auto col = y_.column( c );
          Normalize( col, col.end() );
        }
      }

      /*
        SIMPLS コンストラクタ : 独立変数 X と従属変数 y を指定して構築
      */
      template< class T, class U >
      SIMPLS::SIMPLS( const Matrix< T >& x, const Matrix< U >& y )
      {
        const std::string FUNC_NAME = "tadah_fussy::statistics::regression::SIMPLS constructor";

        if ( x.rows() != y.rows() )
          throw exception::NotEqualLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : data x and y must be same row size.", x.rows(), y.rows() );
        if ( x.rows() == 0 )
          throw exception::NotPositiveLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : row size of x and y must be more than zero.", x.rows() );
        if ( x.cols() == 0 )
          throw exception::NotPositiveLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : column size of x must be more than zero.", x.cols() );
        if ( y.cols() == 0 )
          throw exception::NotPositiveLength< typename Matrix< T >::size_type >
            ( FUNC_NAME + " : column size of y must be more than zero.", y.cols() );

        x_.assign( x.begin(), x.end(), x.rows(), true );
        y_.assign( y.begin(), y.end(), y.rows(), true );

        for ( Matrix< stat_type >::size_type c = 0 ; c < x_.cols() ; ++c ) {
          auto col = x_.column( c );
          Normalize( col, col.end() );
        }
        for ( Matrix< stat_type >::size_type c = 0 ; c < y_.cols() ; ++c ) {
          auto col = y_.column( c );
          Normalize( col, col.end() );
        }
      }

    } // namespace regression

  } // namespace statistics

} // namespace tadah_fussy
